@@ -17,10 +17,8 @@
 ;; General purpose macros which are used in Lisp code, but not widely enough
 ;; accepted to be a part of Lisp systems.
 
-;; 'writefilep' and 'ttyoff' are system independent ways of expressing
-;; the Maclisp ^R and ^W.
+;; 'ttyoff' is a system independent way of expressing the Maclisp ^W.
 
-(defvar writefilep '^r)
 (defvar ttyoff    '^w)
 
 ;; Like PUSH, but works at the other end.
@@ -28,12 +26,6 @@
 (defmacro tuchus (list object)
   `(setf ,list (nconc ,list (ncons ,object))))
 
-;; (EXCH A B) exchanges the bindings of A and B
-;; Maybe it should turn into (PSETF A B B A)?
-
-(defmacro exch (x y)
-  `(setf ,x (prog1 ,y (setf ,y ,x))))
-
 ;; The following macros pertain only to Macsyma.
 
 ;; Except on the Lisp Machine, load the specified macro files.
@@ -51,17 +43,12 @@
   (apply #'load-macsyma-macros-at-runtime macro-files)
   (values))
 
-;; Used to temporarily bind contexts in such a way as to not cause
-;; the context garbage collector to run. Used when you don't want to
-;; stash away contexts for later use, but simply want to run a piece
-;; of code in a new context which will be destroyed when the code finishes.
-;; Note that this code COULD use an unwind-protect to be safe but since
-;; it will not cause out and out errors we leave it out.
-
 (defmacro with-new-context (sub-context &rest forms)
-  `(let ((context (context ,@sub-context)))
-     (prog1 ,@forms
-       (context-unwinder))))
+  `(let ((my-context (gensym "$CTXT")))
+     (mfuncall '$supcontext my-context ,@sub-context)
+     (unwind-protect
+       (prog1 ,@forms)
+       ($killcontext my-context))))
 
 ;; For creating a macsyma evaluator variable binding context.
 ;; (MBINDING (VARIABLES &OPTIONAL VALUES FUNCTION-NAME)
@@ -106,8 +93,8 @@
 (defmacro real-infinityp (x)
   `(member ,x real-infinities :test #'eq))
 
-(defmacro infinityp (x)
-  `(member ,x infinities :test #'eq))
+(defun infinityp (x)
+  (member x infinities :test #'eq))
 
 (defmacro real-epsilonp (x)
   `(member ,x infinitesimals :test #'eq))
@@ -121,6 +108,9 @@
 (defmacro inf-typep (x)
   `(car (amongl infinities ,x)))
 
+(defmacro epsilon-typep (x)
+  `(car (amongl infinitesimals ,x)))
+
 (defmacro hot-coef (p)
   `(pdis (caddr (cadr (rat-no-ratfac ,p)))))
 
@@ -130,7 +120,7 @@
 
 ;; Setf hacking.
 
-(defmfun mget (atom ind)
+(defun mget (atom ind)
   (let ((props (and (symbolp atom) (get atom 'mprops))))
     (and props (getf (cdr props) ind))))
 
@@ -149,7 +139,7 @@
 (defmacro  mdefprop (sym val indicator)
   `(mputprop ',sym ',val ',indicator))
 
-(defmfun mputprop (atom val ind)
+(defun mputprop (atom val ind)
   (let ((props (get atom 'mprops)))
     (if (null props) (putprop atom (setq props (ncons nil)) 'mprops))
     (putprop props val ind)))
