@@ -15,56 +15,91 @@
 (load-macsyma-macros defcal mopers)
 
 (defmvar *alphabet* (list #\_ #\%))
+(defmvar *whitespace-chars*
+         '(#\tab #\space #\linefeed #\return #\page #\newline
+           #+(or unicode sb-unicode openmcl-unicode-strings) #\no-break_space))
 
-(defmfun alphabetp (n)
+(defun alphabetp (n)
   (and (characterp n)
-       (or (alpha-char-p n)
+       (or (alpha-char-p n) #+gcl(>= (char-code n) 128)
 	   (member n *alphabet*))))
 
-(defmfun ascii-numberp (num)
+(defun ascii-numberp (num)
   (and (characterp num) (char<= num #\9) (char>= num #\0)))
 
 (defvar *parse-window* nil)
-(defvar *parse-stream*		()	  "input stream for Maxima parser")
+(defvar *parse-stream* ()     "input stream for Maxima parser")
+(defvar *parse-stream-eof* -1 "EOF value for Maxima parser")
 (defvar *parse-tyi* nil)
 
-(defvar macsyma-operators	()	  "Maxima operators structure")
+(defvar *mread-prompt* nil    "prompt used by `mread'")
+(defvar *mread-eof-obj* ()    "Bound by `mread' for use by `mread-raw'")
+(defvar *current-line-info* nil)
 
-(defvar *mread-prompt*		nil	  "prompt used by `mread'")
-(defvar *mread-eof-obj* () "Bound by `mread' for use by `mread-raw'")
+(defvar *parse-string-input-stream*             ;; reference to the input stream 
+  (let ((stream (make-string-input-stream ""))) ;;   used by parse-string
+    (close stream)                              ;;   in share/stringroc/eval_string.lisp
+    stream ))                                   ;; (see also add-lineinfo below)
+
+(defmvar $report_synerr_line t "If T, report line number where syntax error occurs; otherwise, report FILE-POSITION of error.")
+(defmvar $report_synerr_info t "If T, report the syntax error details from all sources; otherwise, only report details from standard-input.")
 
 (defun mread-synerr (format-string &rest l)
-    (let (tem
-	  errset
-	  (file "stdin"))
-      (errset
-       (setq tem (file-position *parse-stream*))
-       (setq file  (namestring *parse-stream*)))
-      (when tem
-	(format t "~%~a:~a:" file tem))
-      (format t (intl:gettext "incorrect syntax: "))
-      (apply 'format t format-string (mapcar #'(lambda (x)
-						 (if (symbolp x)
-						     (print-invert-case x)
-						     x))
-					     l))
-      (cond ((eql *parse-stream* *standard-input*)
+  (let ((fp (and (not (eq *parse-stream* *standard-input*))
+                 (file-position *parse-stream*)))
+	(file (and (not (eq *parse-stream* *standard-input*))
+                   (cadr *current-line-info*))))
+    (flet ((line-number ()
+	     ;; Fix me: Neither batch nor load track the line number
+	     ;; correctly. batch, via dbm-read, does not track the
+	     ;; line number at all (a bug?).
+	     ;;
+	     ;; Find the line number by jumping to the start of file
+	     ;; and reading line-by-line til we reach the current
+	     ;; position
+	     (cond ((and fp (file-position *parse-stream* 0))
+		    (do ((l (read-line *parse-stream* nil nil) (read-line *parse-stream* nil nil))
+			 (o 1                                  (1+ p))
+			 (p (file-position *parse-stream*)     (file-position *parse-stream*))
+			 (n 1                                  (1+ n)))
+			((or (null p) (>= p fp))
+			 (cons n (- fp o)))))
+		   (t '())))
+	   (column ()
 	     (let ((n (get '*parse-window* 'length))
-		   some ch)
+		   ch some)
 	       (loop for i from (1- n) downto (- n 20)
-		      while (setq ch (nth i *parse-window*))
-		      do
-		      (cond ((eql ch #\newline)
-			     (push #\n some)
-			     (push #\\ some))
-			    ((eql ch #\tab)
-			     (push #\t some)
-			     (push #\\ some))
-			    (t (push ch some))))
+	     	  while (setq ch (nth i *parse-window*))
+		  do
+		    (cond ((char= ch #\newline)
+			   (return-from column some))
+			  (t (push ch some))))
+	       some))
+	   (printer (x)
+	     (cond ((symbolp x)
+		    (print-invert-case (stripdollar x)))
+		   ((stringp x)
+		    (maybe-invert-string-case x))
+		   (t x)))
+	   )
+      (case (and file $report_synerr_line)
+	((t)
+	 ;; print the file, line and column information
+	 (let ((line+column (line-number)))
+	   (format t "~&~a:~a:~a:" file (car line+column) (cdr line+column))))
+	(otherwise
+	 ;; if file=nil, then print a fresh line only; otherwise print
+	 ;; file and character location
+	 (format t "~&~:[~;~:*~a:~a:~]" file fp)))
+      (format t (intl:gettext "incorrect syntax: "))
+      (apply 'format t format-string (mapcar #'printer l))
+      (cond ((or $report_synerr_info (eql *parse-stream* *standard-input*))
+	     (let ((some (column)))
 	       (format t "~%~{~c~}~%~vt^" some (- (length some) 2))
-           (read-line *parse-stream* nil nil))))
+	       (read-line *parse-stream* nil nil))))
       (terpri)
-      (throw-macsyma-top)))
+      (finish-output)
+      (throw-macsyma-top))))
 
 (defun tyi-parse-int (stream eof)
   (or *parse-window*
@@ -95,29 +130,10 @@
 ;;;;;                                                                    ;;;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;; gobble whitespace, recognize '#' comments..
-(defun gobble-whitespace (&aux saw-newline ch saw-other)
-  (do ()
-      (nil) ; Gobble whitespace
-      (setq ch (parse-tyipeek))
-      (cond ((eql ch #\newline)
-	     (setq saw-other nil)
-	     (setq saw-newline t))
-	    ((member ch '(#\tab #\space #\linefeed #\return #\page))
-	     (setq saw-other t))
-	    ;; allow comments to be lines which are whitespace and then
-	    ;; a '#' character.
-	    ;; recognize
-	    ;; # 234 "jim.mac"
-	    ;; to set the current line information to be line 234 of jim.mac
-	    ((and (eql ch #\#) saw-newline)
-	     (let ((li (read-line *parse-stream* nil)))
-	       (unread-char #\newline *parse-stream*)
-	       (setq *parse-tyi* nil)
-	       (unless saw-other
-		   (grab-line-number li *parse-stream*))))
-	    (t  (return t)))
-     (parse-tyi)))
+(defun gobble-whitespace ()
+  (do ((ch (parse-tyipeek) (parse-tyipeek)))
+      ((not (member ch *whitespace-chars*)))
+    (parse-tyi)))
 
 (defun read-command-token (obj)
   (gobble-whitespace)
@@ -148,14 +164,13 @@
 ;; characters, since if you do
 ;; PREFIX("ABCDEFGH");
 ;; then ABCDEFGA should read as a symbol.
-;; 99% of the time we dont have to unparse-tyi, and so there will
+;; 99% of the time we don't have to unparse-tyi, and so there will
 ;; be no consing...
 
-
 (defun parse-tyi ()
   (let ((tem  *parse-tyi*))
     (cond ((null tem)
-	   (tyi-parse-int *parse-stream* -1))
+	   (tyi-parse-int *parse-stream* *parse-stream-eof*))
 	  ((atom tem)
 	   (setq *parse-tyi* nil)
 	   tem)
@@ -167,7 +182,7 @@
 (defun parse-tyipeek ()
   (let ((tem  *parse-tyi*))
     (cond ((null tem)
-	   (setq *parse-tyi* (tyi-parse-int *parse-stream* -1)))
+	   (setq *parse-tyi* (tyi-parse-int *parse-stream* *parse-stream-eof*)))
 	  ((atom tem) tem)
 	  (t (car tem)))))
 
@@ -187,12 +202,12 @@
 (defun read-command-token-aux (obj)
   (let* (result
 	 (ch (parse-tyipeek))
-	 (lis (if (eql ch -1)
+	 (lis (if (eql ch *parse-stream-eof*)
 		  nil
-		  (parser-assoc ch obj))))
+	          (parser-assoc ch obj))))
     (cond ((null lis)
 	   nil)
-	  (t
+          (t
 	   (parse-tyi)
 	   (cond ((atom (cadr lis))
 		  ;; INFIX("ABC"); puts into macsyma-operators
@@ -200,16 +215,25 @@
 		  ;; ordinary things are like:
 		  ;; (#\< (ANS $<) (#\= (ANS $<=)))
 		  ;; where if you fail at the #\< #\X
-		  ;; stage, then the previous step was permitted.
-		  (setq result (read-command-token-aux (list (cdr lis) ))))
-		 ((null (cddr lis))
+	          ;; stage, then the previous step was permitted.
+		  (setq result (read-command-token-aux (list (cdr lis)))))
+	         ((null (cddr lis))
 		  ;; lis something like (#\= (ANS $<=))
 		  ;; and this says there are no longer operators
 		  ;; starting with this.
 		  (setq result
-			(and (eql (car (cadr lis)) 'ans)
-			     (cadr (cadr lis)))))
-		 (t
+		        (and (eql (car (cadr lis)) 'ans)
+		             ;; When we have an operator, which starts with a
+		             ;; literal, we check, if the operator is
+		             ;; followed with a whitespace. With this code
+		             ;; Maxima parses an expression grad x or grad(x)
+		             ;; as (($grad) x) and gradef(x) as (($gradef) x),
+		             ;; when grad is defined as a prefix operator.
+		             ;; See bug report ID: 2970792.
+		             (or (not (alphabetp (cadr (exploden (cadr (cadr lis))))))
+		                 (member (parse-tyipeek) *whitespace-chars*))
+		             (cadr (cadr lis)))))
+	         (t
 		  (let ((res   (and (eql (car (cadr lis)) 'ans)
 				    (cadr (cadr lis))))
 			(com-token (read-command-token-aux (cddr lis) )))
@@ -218,7 +242,6 @@
 	     (or result (unparse-tyi ch))
 	     result))))
 
-
 (defun scan-macsyma-token ()
   ;; note that only $-ed tokens are GETALIASed.
   (getalias (implode (cons '#\$ (scan-token t)))))
@@ -240,9 +263,13 @@
 (defun scan-token (flag)
   (do ((c (parse-tyipeek) (parse-tyipeek))
        (l () (cons c l)))
-      ((and flag (not (or (digit-char-p c (max 10. *read-base*)) (alphabetp c) (char= c #\\))))
-       (nreverse (or l (ncons (parse-tyi))))) ; Read at least one char ...
-    (when (char= (parse-tyi) #\\)
+      ((or (eql c *parse-stream-eof*)
+           (and flag
+                (not (or (digit-char-p c (max 10 *read-base*))
+                         (alphabetp c)
+                         (char= c #\\ )))))
+       (nreverse (or l (list (parse-tyi))))) ; Read at least one char ...
+    (when (char= (parse-tyi) #\\ )
       (setq c (parse-tyi)))
     (setq flag t)))
 
@@ -255,11 +282,11 @@
     (when init
       (vector-push-extend init buf))
     (do ((c (parse-tyipeek) (parse-tyipeek)))
-	((cond ((eql c -1))
+	((cond ((eql c *parse-stream-eof*))
 	       ((char= c #\")
 		(parse-tyi) t))
 	 (copy-seq buf))
-      (if (char= (parse-tyi) #\\)
+      (if (char= (parse-tyi) #\\ )
 	  (setq c (parse-tyi)))
       (vector-push-extend c  buf))))
 
@@ -305,6 +332,7 @@
 			 (map nil #'princ x))
 		 (reverse numlist))
 	    (terpri)
+	    (finish-output)
 	    (unless (equalp true fast)
 	      (incf failures)
 	      (format t "NUM:  ~A~%  TRUE: ~S~%  FAST: ~S~%"
@@ -326,7 +354,9 @@
         (setf (nth 3 data) (list flonum-exponent-marker)))))
   (if (not (equal (nth 3 data) '(#\B)))
       (readlist (apply #'append data))
-      (let ((int-part (readlist (or (first data) '(#\0))))
+      (let*
+	   ((*read-base* 10.)
+	    (int-part (readlist (or (first data) '(#\0))))
 	    (frac-part (readlist (or (third data) '(#\0))))
 	    (frac-len (length (third data)))
 	    (exp-sign (first (fifth data)))
@@ -338,7 +368,7 @@
 	    ;; For example, 1.234b1000 is converted by computing
 	    ;; bfloat(1234)*10b0^(1000-3).  Extra precision is used
 	    ;; during the bfloat computations.
-	    (let* ((extra-prec (+ *fast-bfloat-extra-bits* (ceiling (log exp 2d0))))
+	    (let* ((extra-prec (+ *fast-bfloat-extra-bits* (ceiling (log exp 2e0))))
 		   (fpprec (+ fpprec extra-prec))
 		   (mant (+ (* int-part (expt 10 frac-len)) frac-part))
 		   (bf-mant (bcons (intofp mant)))
@@ -390,13 +420,13 @@
 	      ;; We're trying to parse the exponent part of a number,
 	      ;; and we didn't get a value after the exponent marker.
 	      ;; That's an error.
-	      (merror (intl:gettext "parser: incomplete number; missing exponent?")))
+	      (mread-synerr "parser: incomplete number; missing exponent?"))
 	     (t
 	      (make-number (cons (nreverse l) data)))))
     (parse-tyi)))
 
 (defun scan-number-after-dot (data)
-  (scan-digits data '(#\E #\e #\F #\f #\B #\b #\D #\d #\S #\s) #'scan-number-exponent))
+  (scan-digits data '(#\E #\e #\F #\f #\B #\b #\D #\d #\S #\s #\L #\l #+cmu #\W #+cmu #\w) #'scan-number-exponent))
 
 (defun scan-number-exponent (data)
   (push (ncons (if (or (char= (parse-tyipeek) #\+)
@@ -490,19 +520,21 @@
   (scan-one-token-g nil nil))
 
 (defun scan-one-token-g (eof-ok? eof-obj)
+  (declare (special macsyma-operators))
   (cond ((car scan-buffered-token)
 	 (rplaca scan-buffered-token ())
 	 (cdr scan-buffered-token))
 	((read-command-token macsyma-operators))
 	(t
 	 (let ((test (parse-tyipeek)))
-	   (cond  ((eql test -1.)
+	   (cond  ((eql test *parse-stream-eof*)
 		   (parse-tyi)
 		   (if eof-ok? eof-obj
 		       (maxima-error (intl:gettext "parser: end of file while scanning expression."))))
 		  ((eql test #\/)
 		   (parse-tyi)
 		   (cond ((char= (parse-tyipeek) #\*)
+                          (parse-tyi)
 			  (gobble-comment)
 			  (scan-one-token-g eof-ok? eof-obj))
 			 (t '$/)))
@@ -535,7 +567,8 @@
 	(setq c (parse-tyipeek))
 	(parse-tyi)
 	(cond ((= depth 0) (return t)))
-	(cond ((and (numberp c) (< c 0))(error (intl:gettext "parser: end of file in comment.")))
+	(cond ((eql c *parse-stream-eof*)
+	       (error (intl:gettext "parser: end of file in comment.")))
 	      ((char= c #\*)
 	       (cond ((char= (parse-tyipeek) #\/)
 		      (decf depth)
@@ -551,10 +584,10 @@
 
 (defun scan-number-rest (data)
   (let ((c (caar data)))
-    (cond ((member c '(#\.))
+    (cond ((char= c #\.)
 	   ;; We found a dot
 	   (scan-number-after-dot data))
-	  ((member c '(#\E #\e #\F #\f #\B #\b #\D #\d #\S #\s))
+	  ((member c '(#\E #\e #\F #\f #\B #\b #\D #\d #\S #\s #\L #\l #+cmu #\W #+cmu #\w))
 	   ;; Dot missing but found exponent marker.  Fake it.
 	   (setf data (push (ncons #\.) (rest data)))
 	   (push (ncons #\0) data)
@@ -562,7 +595,7 @@
 	   (scan-number-exponent data)))))
 
 (defun scan-number-before-dot (data)
-  (scan-digits data '(#\. #\E #\e #\F #\f #\B #\b #\D #\d #\S #\s)
+  (scan-digits data '(#\. #\E #\e #\F #\f #\B #\b #\D #\d #\S #\s #\L #\l #+cmu #\W #+cmu #\w)
 	       #'scan-number-rest))
 
 
@@ -644,20 +677,20 @@
 (defmacro def-nud ((op . lbp-rbp) bvl . body)
   (let (( lbp (nth 0 lbp-rbp))
 	( rbp (nth 1 lbp-rbp)))
-    `(progn 'compile 	  ,(make-parser-fun-def op 'nud bvl body)
+    `(progn ,(make-parser-fun-def op 'nud bvl body)
 	    (set-lbp-and-rbp ',op ',lbp ',rbp))))
 
 (defun set-lbp-and-rbp (op lbp rbp)
   (cond ((not (consp op))
 	 (let ((existing-lbp (get op 'lbp))
 	       (existing-rbp (get op 'rbp)))
-	   (cond ((not lbp) ;; ignore ommitted arg
+	   (cond ((not lbp) ;; ignore omitted arg
 		  )
 		 ((not existing-lbp)
 		  (putprop op lbp 'lbp))
 		 ((not (equal existing-lbp lbp))
 		  (maxima-error "Incompatible LBP's defined for this operator ~a" op)))
-	   (cond ((not rbp) ;; ignore ommitted arg
+	   (cond ((not rbp) ;; ignore omitted arg
 		  )
 		 ((not existing-rbp)
 		  (putprop op rbp 'rbp))
@@ -683,8 +716,7 @@
 (defmacro def-led((op . lbp-rbp) bvl . body)
   (let (( lbp (nth 0 lbp-rbp))
 	( rbp (nth 1 lbp-rbp)))
-    `(progn 'compile
-	    ,(make-parser-fun-def  op 'led bvl body)
+    `(progn ,(make-parser-fun-def  op 'led bvl body)
 	    (set-lbp-and-rbp ',op ',lbp ',rbp))))
 
 (defmacro def-collisions (op &rest alist)
@@ -692,7 +724,7 @@
 		   (lis  alist (cdr lis))
 		   (nl ()    (cons (cons (caar lis) i) nl)))
 		  ((null lis) nl))))
-    `(progn 'compile
+    `(progn
        (defprop ,op ,(let nil
 			  (copy-tree keys )) keys)
        ,@(mapcar #'(lambda (data)
@@ -736,7 +768,7 @@
 ;;; (LBP <op>)		 - reads an operator's Left Binding Power
 ;;; (DEF-LBP <op> <val>) - defines an operator's Left Binding Power
 
-(defmfun lbp (lex) (cond ((safe-get lex 'lbp)) (t 200.)))
+(defun lbp (lex) (cond ((safe-get lex 'lbp)) (t 200.)))
 
 (defmacro def-lbp (sym val) `(defprop ,sym ,val lbp))
 
@@ -745,7 +777,7 @@
 ;;; (RBP <op>)		 - reads an operator's Right Binding Power
 ;;; (DEF-RBP <op> <val>) - defines an operator's Right Binding Power
 
-(defmfun rbp (lex) (cond ((safe-get lex 'rbp)) (t 200.)))
+(defun rbp (lex) (cond ((safe-get lex 'rbp)) (t 200.)))
 
 (defmacro def-rbp (sym val) `(defprop ,sym ,val rbp))
 
@@ -784,14 +816,14 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
 ;;;; Misplaced definitions
 
 (defmacro def-operatorp ()
-  `(defmfun operatorp (lex)
+  `(defun operatorp (lex)
      (and (symbolp lex) (getl lex '(,@(nud-propl) ,@(led-propl))))))
 
 (def-operatorp)
 
 (defmacro def-operatorp1 ()
   ;Defmfun -- used by SYNEX if not others.
-  `(defmfun operatorp1 (lex)
+  `(defun operatorp1 (lex)
      ;; Referenced outside of package: OP-SETUP, DECLARE1
      ;; Use for truth value only, not for return-value.
      (and (symbolp lex) (getl lex '(lbp rbp ,@(nud-propl) ,@(led-propl))))))
@@ -806,12 +838,10 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
 ;;;
 ;;; If you want rubout processing, be sure to call some stream which knows
 ;;; about such things. Also, I'm figuring that the PROMPT will be
-;;; an atribute of the stream which somebody can hack before calling
+;;; an attribute of the stream which somebody can hack before calling
 ;;; MREAD if he wants to.
 
 
-(defvar *current-line-info* nil)
-
 ;;Important for lispm rubout handler
 (defun mread (&rest read-args)
   (progn
@@ -819,7 +849,7 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
       (and *parse-window* (setf (car *parse-window*) nil
 				*parse-window* (cdr *parse-window*)))
       (princ *mread-prompt*)
-      (force-output))
+      (finish-output))
     (apply 'mread-raw read-args)))
 
 (defun mread-prompter (stream char)
@@ -1119,9 +1149,19 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
     (cond ((eq '|$)| (first-c)) (parse-err))		  ; () is illegal
 	  ((or (null (setq right (prsmatch '|$)| '$any))) ; No args to MPROGN??
 	       (cdr right))				  ;  More than one arg.
+	  (when (suspicious-mprogn-p right)
+	    (mtell (intl:gettext "warning: parser: I'll let it stand, but (...) doesn't recognize local variables.~%"))
+	    (mtell (intl:gettext "warning: parser: did you mean to say: block(~M, ...) ?~%") (car right)))
 	   (cons '$any (cons hdr right)))	  ; Return an MPROGN
 	  (t (cons '$any (car right))))))		  ; Optimize out MPROGN
 
+(defun suspicious-mprogn-p (right)
+  ;; Look for a Maxima list of symbols or assignments to symbols.
+  (and ($listp (car right))
+       (every #'(lambda (e) (or (symbolp e)
+                                (and (consp e) (eq (caar e) 'msetq) (symbolp (second e)))))
+              (rest (car right)))))
+
 (def-led (|$(| 200.) (op left)
   (setq left (convert left '$any))		        ;De-reference LEFT
   (if (numberp left) (parse-err))			;number(...) illegal
@@ -1141,7 +1181,7 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
     (cond ((eq '|$(| (first-c))
 	   (list '$any (mheader '|$'|) (parse '$any 190.)))
 	  ((or (atom (setq right (parse '$any 190.)))
-	       (member (caar right) '(mquote mlist mprog mprogn lambda) :test #'eq))
+	       (member (caar right) '(mquote mlist $set mprog mprogn lambda) :test #'eq))
 	   (list '$any (mheader '|$'|) right))
 	  ((eq 'mqapply (caar right))
 	   (cond ((eq (caaadr right) 'lambda)
@@ -1672,6 +1712,12 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
 ;; they are all generic common lisp and could be used by
 ;; any Common lisp implementation.
 
+#-gcl
+(defstruct instream
+  stream
+  (line 0 :type fixnum)
+  stream-name)
+
 #-gcl
 (defvar *stream-alist* nil)
 
@@ -1685,12 +1731,6 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
   (or (instream-stream-name instr)
       (stream-name (instream-stream instr))))
 
-#-gcl
-(defstruct instream
-  stream
-  (line 0 :type fixnum)
-  stream-name)
-
 ;; (closedp stream) checks if a stream is closed.
 ;; how to do this in common lisp!!
 
@@ -1723,9 +1763,12 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
 
 
 (defun add-lineinfo (lis)
-  (if (or (atom lis) (and (eq *parse-window* *standard-input*)
-			  (not (find-stream *parse-stream*))))
-			  lis
+  (if (or (atom lis) 
+          (eq *parse-stream* *parse-string-input-stream*) ;; avoid consing *parse-string-input-stream* 
+                                                          ;;   via get-instream to *stream-alist* 
+          (and (eq *parse-window* *standard-input*)
+               (not (find-stream *parse-stream*)) ))
+    lis
     (let* ((st (get-instream *parse-stream*))
  	   (n (instream-line st))
 	   (nam (instream-name st)))
@@ -1739,3 +1782,17 @@ entire input string to be printed out when an MAXIMA-ERROR occurs."
       (cond ((null (cdr lis))
 	     (list (car lis) *current-line-info*))
 	    (t (append lis (list *current-line-info*)))))))
+
+;; Remove debugging stuff.
+;; STRIP-LINEINFO does not modify EXPR.
+
+(defun strip-lineinfo (expr)
+  (if (atom expr) expr
+    (cons (strip-lineinfo-op (car expr)) (mapcar #'strip-lineinfo (cdr expr)))))
+
+;; If something in the operator looks like debugging stuff, remove it.
+;; It is assumed here that debugging stuff is a list comprising an integer and a string
+;; (and maybe other stuff, which is ignored).
+
+(defun strip-lineinfo-op (maxima-op)
+  (remove-if #'(lambda (x) (and (consp x) (integerp (first x)) (stringp (second x)))) maxima-op))
